/********** GENERADOR DEL ALGORITMO NUMBERKN VERSION EXHAUSTIVA PARA EL ALGORITMO DE PERES DE DIMENSION N **************/

#include <stdio.h>
#include <string.h>

#define MAXCAR 256

/**
Este programa escribe el código en C del algoritmo comprueba que, dado un grafo, todos sus vértices estén 
contenidos en un Kn, siendo n la dimensión considerada para el problema.

Alberto Solís Encina. Copyright 2011. Histórico:
			13-12-2010: - Añadidos variables genéricas y parámetros del algoritmo.
			13-12-2010: - Modificado para que devuelva una estructura definida como STRUCTKN
			27-12-2010: - Utilización de funciones de manipulación del fichero propias de C.
			30-12-2010: - Tipo del main modificado a 'int' para devolver código de error.
			30-12-2010: - Código compilado correctamente.
			07-01-2011: - Extraída la definición de la estructura StructInKn en "AlgPeres.h"
			07-01-2011: - Uso de DYNALLOC1 para almacenamiento dinámico de los arrays de la estructura.
			07-04-2011: - Cambio al uso de malloc para almacenamiento de los arrays con tamaño precalculado.
					Ya no se utiliza DYNALLOC1 en esta versión.

**/

/** main devuelve -1 en caso de error, devuelve 0 en cualquier otro caso. **/
int main (int argc, int argv[]){
			/** Pasamos como argumento la dimensión con la que trabajar (dim)**/
			long dim =(long)atoi(argv[1]);
			//Varbiables del programa
			int contador=0;
			char s [MAXCAR];
			FILE* f;
			char variable_indice = 'a'; //usaremos como nombres de los arryas de la estructura usando de la a a la z (siguiendo el código ASCII).

			//Apertura de fichero
			if ((f=fopen("numberKn.c","w")) == NULL){
				perror("Error al abrir el archivo f");
				return -1;}

		sprintf(s,"/** numberKn para dimensión %ld **/\n",dim);	
		fputs(s, f); //uso de formato de texto
		sprintf(s,"/** Alberto Solis Encina. Copyright 2011. MA1.**/\n");
		fputs(s, f);

		sprintf(s, "/** Calcula el número de Kn, con n la dimensión del problema, contenidos en g (versión exhaustiva) **/ \n");
		fputs(s, f);

		sprintf(s, "#include <stdio.h>\n#include \"AlgPeres.h\"\n\n");
		fputs(s,f);

		sprintf(s,"/** Resuelve el problema de cálculo del número de K%d en un grafo **/\n\n",dim);
		fputs(s, f);
			
		//-----------------------------------------------------------------------------------------
		sprintf(s,"STRUCTKN numberKn (graph* g, int m, int n){ \n\n");
		fputs(s,f);
		
		 //*************************** Declaracion de variables ***********************************
		
		//Deben declararse por tanto el mismo número de variables como dimension
		 for(contador=0;contador<dim;contador++){
			sprintf(s,"int var%d;\n", contador);
			fputs(s, f);
		 }
		
		 //Deben declararse tantos setwords como número de 'while' menos el último que aparecen y que almacenan los vecinos adyacentes
		 for(contador=0;contador<dim-1;contador++){
			sprintf(s,"setword ady%d;\n",contador);
			fputs(s, f);
		 }	

		sprintf(s, "int nks=0;\n");
		fputs(s,f);
		
		 //Inicialización de los valores de la estructura
		 sprintf(s,"\nSTRUCTKN str = NULL;\nstr = (STRUCTKN) malloc (sizeof(struct StructInKn));\nstr->nKn = 0;\n");
		 fputs(s, f);
		
		//Inicialización de los punteros a arrays
		 variable_indice = 'a';
		 for(contador=0;contador<dim;contador++){
		 	sprintf(s,"str->T%c=NULL;\n", variable_indice);
		 	fputs(s, f);
			variable_indice++;
		}

		sprintf(s,"\n\n");
		fputs(s, f);
		 
						
		//********************************** Precálculo del número de Kns *******************************************
		
		sprintf(s,"\n/** Precálculo del número de Kns **/\n\n");
		fputs(s, f);

		//Raíz del anidamiento de bucles
		sprintf(s,"\nfor (var0=0;var0<n-%ld+1;var0++){\n ady0 = g[var0] & BITMASK(var0);\n\tif(POPCOUNT(ady0)<%ld) ady0=0;\n",dim,dim-1);
		fputs(s, f);
		
		
		//Bucle 'for' para generar el conjunto de bucles anidados
		for(contador=1;contador<=dim-3;contador++){
			sprintf(s, "while(ady%d){\n TAKEBIT(var%d,ady%d);\n ady%d=g[var%d] & ady%d;\n\tif(POPCOUNT(ady%d)<%ld) ady%d=0;\n\n",contador-1,contador,contador-1,contador,contador,contador-1, contador, dim-1-contador, contador);
			fputs(s, f);
		}
		
		sprintf(s,"while(ady%ld){\n\t TAKEBIT(var%ld,ady%ld);\n\t ady%ld = g[var%ld] & ady%d;\n",dim-3,dim-2,dim-3,dim-2,dim-2,dim-3);
		fputs(s, f);
		
		sprintf(s,"while(ady%ld){\n\t TAKEBIT(var%ld,ady%ld);\n",dim-2,dim-1,dim-2);
		fputs(s, f);

		
		sprintf(s,"\tnks= nks + 1;\n");
		fputs(s, f);
		
		for(contador=0;contador<dim;contador++){
			sprintf(s,"}\n");
			fputs(s, f);
		}
		

		 /************************************ Alojamiento en memoria de los arrays Tx ************************/

		 sprintf(s,"\n/** Declaramos los tamaños de los arrays según el valor calculado en 'nks' **/\n\n");
		 fputs(s, f);


		 variable_indice = 'a';
		 for(contador = 0; contador < dim; contador++){
			sprintf(s,"str->T%c = (int*) malloc(sizeof(int)*nks);\n",variable_indice);
			fputs(s, f);
			variable_indice++;
		 }

		/*********************************** Búsqueda de los vértices que forman los Kns **************************/
	
		 sprintf(s,"\n/** Búsqueda y almacenamiento de los vértices que forman los Kns **/\n\n");
		 fputs(s, f);		

		//Raíz del anidamiento de bucles
		sprintf(s,"\nfor (var0=0;var0<n-%ld+1;var0++){\n ady0 = g[var0] & BITMASK(var0);\n\tif(POPCOUNT(ady0)<%ld) ady0=0;\n",dim,dim-1);
		fputs(s, f);
		
		
		//Bucle 'for' para generar el conjunto de bucles anidados
		for(contador=1;contador<=dim-3;contador++){
			sprintf(s, "while(ady%d){\n TAKEBIT(var%d,ady%d);\n ady%d=g[var%d] & ady%d;\n\tif(POPCOUNT(ady%d)<%ld) ady%d=0;\n\n",contador-1,contador,contador-1,contador,contador,contador-1, contador, dim-1-contador, contador);
			fputs(s, f);
		}
		
		sprintf(s,"while(ady%ld){\n\t TAKEBIT(var%ld,ady%ld);\n\t ady%ld = g[var%ld] & ady%d;\n",dim-3,dim-2,dim-3,dim-2,dim-2,dim-3);
		fputs(s, f);
		
		sprintf(s,"while(ady%ld){\n\t TAKEBIT(var%ld,ady%ld);\n",dim-2,dim-1,dim-2);
		fputs(s, f);
		
		variable_indice='a';
		for(contador=0;contador<dim;contador++){
			sprintf(s,"\tstr->T%c[str->nKn]=var%d;\n",variable_indice,contador);
			fputs(s, f);
			variable_indice++;
		}
		
		sprintf(s,"\tstr->nKn= str->nKn+1;\n");
		fputs(s, f);
		
		for(contador=0;contador<dim;contador++){
			sprintf(s,"}\n");
			fputs(s, f);
		}
		


		/***************** Retorno de la función ***********************/
		sprintf(s,"\nreturn str;\n}");
		fputs(s, f);
				
		fclose(f);
return 0;
}
